       SUBROUTINE MinFOSangle
     
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!      This subroutine uses the secant method to determine slide angle for iteration
!      to minimize factor of safety.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Called by Fos
!
!      VARIABLES  
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

      USE CommonData, ONLY: dp,slipdirtol
      USE GridData, ONLY: invPiover180,Piover180,angle,angslope,cmang,pi
      USE SetData, ONLY: cosang,sinang
      USE FOSData, ONLY: nangle,newangle,foso,felfoso
      
      IMPLICIT NONE

      REAL(dp) :: fosprime1,fosprime2,pcntdiff1,pcntdiff2,felsave1,felsave2
      REAL(dp) :: fos1,fos2,angnew,angdiff1,angdiff2,angdiff3,angdiffcm
      
      INTEGER :: cmflag
      
      LOGICAL :: angdiffbig

      SAVE fos1,fos2,fosprime1,fosprime2,angdiff1,angdiff2,cmflag,felsave1,felsave2,&
           pcntdiff1,angdiffcm

      newangle = 1
      IF (nangle.eq.1) THEN
        angdiff2 = 0._dp
        angdiff1 = -Piover180
        angnew = angslope + angdiff1
        cosang = COS(angnew)
        sinang = SIN(angnew)
        fos2 = foso
        felsave1 = felfoso
        RETURN
      ELSE IF (nangle.eq.2) THEN
        pcntdiff1 = (foso - fos2) / fos2
        fosprime1 = (fos2 - foso) / (angdiff2 - angdiff1)
        angdiff2 = Piover180
        angdiff1 = 0._dp
        angnew = angslope + angdiff2
        cosang = COS(angnew)
        sinang = SIN(angnew)
        fos1 = fos2  !  slope angle FOS
        fos2 = foso  !  FOS at minus one degree 
        felsave2 = felfoso
        cmflag = 0
!     Find difference between slope direction and center of mass slide direction.   
        angdiffcm = cmang-angslope
        RETURN
      ELSE 
        IF (nangle.eq.3) THEN
          pcntdiff2 = (foso - fos2) / fos2
!     If FOS on either side of slope angle is bigger and within tolerance
!     stop angle iteration and exit
          IF (pcntdiff1.gt.0._dp.and.pcntdiff2.gt.0._dp.and.&
              MIN(pcntdiff1,pcntdiff2).lt.slipdirtol) THEN
            foso = fos1
            felfoso = felsave1
            newangle = 0
            RETURN
          END IF
        END IF
      
!     Use secant method to find next slide angle, using difference in angle from original slope angle as 
!     independent variable and limit to +/- pi
        IF (nangle.le.10) THEN
          fosprime2 = (foso - fos1) / (angdiff2 - angdiff1)
          angdiff3 = angdiff2 - fosprime2 * (angdiff2 - angdiff1)/(fosprime2 - fosprime1)
!     Limit change in angle to +/- pi
          IF (ABS(angdiff3).gt.pi) angdiff3 = SIGN(1._dp,angdiff3) * pi
          angnew = angdiff3 + angslope
!        Print *,(angslope+angdiff1)*invPiover180,(angslope+angdiff2)*invPiover180,fos1,foso,felsave1,felfoso
          IF (nangle.eq.3) THEN
!     If new (4th) angle is less than the initial angle save angle and derivative parameters associated with slope angle
            IF (angdiff3.lt.0._dp) THEN
              angdiff2 = -Piover180
              fosprime2 = fosprime1
            ELSE
              fos2 = foso
              felsave2 = felfoso
            END IF
          ELSE
            fos2 = foso
            felsave2 = felfoso
          END IF
          angdiffbig = ABS(angdiff3-angdiff2) - 0.001_dp*Piover180 > 0._dp
!     Check whether new angle is bound by last angle tried and redefine bounding parameters if not
          IF (SIGN(1._dp,angdiff2).eq.SIGN(1._dp,angdiff3).and.ABS(angdiff3).gt.ABS(angdiff2)&
               .and.angdiffbig) THEN
            fos1 = fos2
            fosprime1 = fosprime2
            angdiff1 = angdiff2
            felsave1 = felsave2
          ELSE  ! new angle is between last two angles so check percent change in FOS
            pcntdiff1 = (fos1 - fos2) / fos1
            IF (ABS(pcntdiff1).lt.slipdirtol.or.(.NOT.angdiffbig)) THEN
              IF (foso.lt.fos1) THEN
                angle = (angslope + angdiff2) * invPiover180
              ELSE
                angle = (angslope + angdiff1) * invPiover180
                foso = fos1
                felfoso = felsave1
              END IF
              IF (angle.lt.0.0_dp) angle = angle + 360.0_dp              
              newangle = 0
              RETURN
            END IF
            IF (cmflag.eq.0) THEN
              IF (SIGN(1._dp,angdiffcm).eq.SIGN(1._dp,angdiff3)) THEN
                IF (ABS(angdiffcm).lt.ABS(angdiff3).and.ABS(angdiffcm).gt.ABS(angdiff2)) THEN
                  angdiff3 = angdiffcm
                  cmflag = 1
                END IF
              ELSE
                cmflag = 1
              END IF
            END IF
          END IF
          angdiff2 = angdiff3
          cosang = COS(angnew)
          sinang = SIN(angnew)
        ELSE  !  exceeded 10 iterations so save minimum FOS and associated angle
          IF (foso.lt.fos1) THEN
            angle = (angslope + angdiff2) * invPiover180
          ELSE
            angle = (angslope + angdiff1) * invPiover180
            foso = fos1
            felfoso = felsave1
          END IF
          IF (angle.lt.0.0_dp) angle = angle + 360.0_dp 
          newangle = 0
        ENDIF
      END IF
      
      END SUBROUTINE MinFOSangle