        SUBROUTINE SubsetError(ierr)
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine writes error messages regarding exceedence of
!       maximum number of land or water subsets.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!      Called by Scoops3D
!
!      VARIABLES
!
!        ierr -- error flag indicating initial radius problem (2) 
!           or too many subsets (1)
!        nnset -- maximum number of subsets
!        rad -- search spheroid axes perpendicular to strike of slope
!        subsetwarn -- Flag for limiting number of statements in error file
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE GridData, ONLY: xcen,ycen,zcen,rad
        USE SetData, ONLY: nnset,subsetwarn
        
        IMPLICIT NONE
        
        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
          
          SUBROUTINE WriteError2(errtype,problem,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problem,solution
          END SUBROUTINE WriteError2
        END INTERFACE
        
        INTEGER, INTENT(in) :: ierr
        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage,solution
        INTEGER :: nnnset
        
        IF (ierr.eq.1) THEN
          errmessage = 'exceeded maximum number of subsets'
          solution = 'decrease DEM resolution or increase nnset in modules.f90' 
          problemtype = 'exceeded maximum number of subsets'
          nnnset = nnset
          PRINT *,'** greater than ',nnnset,' trial surfaces at search node, x = ', &
            xcen,'y = ',ycen,'z = ',zcen,'rad = ',rad
          WRITE (39,*) '** greater than ',nnnset,' trial surfaces at search node, x = ', &
            xcen,'y = ',ycen,'z = ',zcen
        END IF                       
        IF (subsetwarn.eq.0) THEN ! on first occurrence, write warning to summary file and error file
          Call WriteError(0,errmessage,problemtype,solution,'yes',0,' ')
          subsetwarn = 1 
        ELSE ! write warning to error file only
          Call WriteError2(0,errmessage,solution,'yes',0,' ')
        END IF       
        PRINT *,'Scoop3D will continue to next axis ratio or next search node'       
        WRITE (39,*) 'Scoop3D will continue to next axis ratio or next search node'


        END SUBROUTINE SubsetError