      SUBROUTINE WriteCritLattice
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!     This subroutine writes critlattice file - file of locations of all
!     search centers, with factor of safety written only for critical
!     centers, 10000 for noncritical centers with FOS < 100, 9999 for no 
!     surfaces found at center, 111 for nonconvergent centers, and 
!     100 for factor of safety > 100.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!     Called by Writeout
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx     

        USE CommonData, ONLY: dp
        USE GridData, ONLY: nx,ny,iretro
        USE SearchData, ONLY: nrec,linenum
        USE FOSData, ONLY: fsvol,icritlattice,nretro,single,foscut,ofos
        
        IMPLICIT NONE

        INTERFACE
          SUBROUTINE WriteError(errtype,problem,problemtype,solution,extraflag,extra1int,extra1ch)
            INTEGER, INTENT(in) :: errtype,extra1int
            CHARACTER(len=*), INTENT(in) :: extraflag,extra1ch,problemtype,problem,solution
          END SUBROUTINE WriteError
        END INTERFACE
        
        INTEGER :: i,j,k,kk,n,nn,nlast,ncount,error,critflag
        INTEGER, ALLOCATABLE :: linelist(:)

        REAL(dp) :: x,y,z,f
        
        CHARACTER(70)  :: problemtype
        CHARACTER(120)  :: errmessage,solution
        CHARACTER(10) :: header

        error = 0

!     Deallocate an array to ensure there is enough memory for new array
        DEALLOCATE(fsvol)
        ALLOCATE (linelist(nx*ny),STAT = error)
        
        IF (error.ne.0) THEN
          errmessage = 'array not allocated successfully'
          problemtype = 'WriteCritLattice'
          solution = 'reduce memory requirements. See "Practical Considerations" chapter of Scoops3D manual'
          Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')         
        END IF
        
        nlast = 0
        ncount = 0
!     Loop through linenum array to create list of unique line numbers of file of all search centers
!     to make list of unique critical centers.
        DO j = 1,ny
          DO i = 1,nx
            n = linenum(i,j)
!     Compare to prior search center line before looping to save time because
!     there may be a number of equivalent centers in a row
            IF (n.eq.nlast.or.n.eq.0) CYCLE
            nlast = 0
!     Compare current line number to list to see if new and add if so
            IF (ncount.gt.0) THEN 
              DO k = 1,ncount
                IF (n.eq.linelist(k)) THEN
                  nlast = n
                  EXIT
                END IF
              END DO
            END IF
            IF (nlast.eq.0) THEN
              nlast = n
              ncount = ncount + 1
              IF (ncount.eq.1) THEN
                linelist(1) = n
              ELSE
!     Insert into array in sort order from lowest to highest record number
                DO kk = ncount-1,1,-1
                  IF (n.lt.linelist(kk)) THEN
                    linelist(kk+1) = linelist(kk)
                    IF (kk.eq.1) linelist(kk) = n
                  ELSE
                    linelist(kk+1) = n
                    EXIT
                  END IF
                END DO
              END IF
            END IF
          END DO
        END DO

!     Read file of all search centers and determine which lines are associated
!     with critical slides on the ground using list of unique line number created above.  
!     If printing all search centers but only writing FOS for critical centers (lcritlattice=1)
!     adjust FOS according to print rules. Otherwise (icritlattice=2) only print critical centers.
        REWIND(15)
        READ (15,*) header
        nn = 1
        DO k = 1,nrec-1
          READ (15,*) x,y,z,f
          IF (nn.le.ncount) THEN
            IF (k.eq.linelist(nn)) THEN
              nn = nn + 1
              critflag = 1
            ELSE 
              IF (f.lt.100.0) f = 10000.0
              critflag = 0
            END IF 
          ELSE
            IF (f.lt.100.0) f = 10000.0
            critflag = 0
          END IF
!     Write critical factor of safety if found on ground (k.eq.linelist(nn)) or
!     write 10,000 if not critical and FOS<100, or other value signaling 
!     nonconvergence (111), no values met requirements (9999), or excessively high FOS (100)
!     unless icritlattice=2 and only writing critical centers.                
          IF (critflag.eq.1.or.icritlattice.eq.1) WRITE (45,1000) x,y,z,f
        END DO    

        IF (nretro.gt.0.and.iretro.lt.nretro.and.ofos.lt.foscut.and.single.eq.0) THEN
          DEALLOCATE (linelist)
          ALLOCATE (fsvol(nx,ny),STAT=error)
          IF (error.ne.0) THEN
            errmessage = 'arrays not allocated successfully'
            problemtype = 'WriteCritLattice'            
            solution = 'reduce memory requirements. See "Practical Considerations" chapter of Scoops3D manual'
            Call WriteError(1,errmessage,problemtype,solution,'no ',0,' ')         
          END IF
        END IF
        
        CLOSE(15)
          
1000    FORMAT (3ES24.17,f11.4)

        END SUBROUTINE WriteCritLattice
