        SUBROUTINE checksets(i,j) 
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine determines set membership of columns based on
!       included nodes determined in subroutine floodfill.  Land nodes
!       take precedence for assigning column set membership.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!        Called by Spheresets
!
!        VARIABLES
!
!        i,j -- DEM column number
!        ibdy -- flag marking column as on DEM boundary
!        inn(nx,ny) -- number of nodes of each column bounded by spheroid
!        inscoop(nx,ny) -- indicates subset membership of DEM column nodes
!        jj -- nodes of DEM column
!        m -- set number
!        mini(ntotset),maxi(ntotset) -- minimum and maximum i bounds of each subset
!        minj(ntotset),maxj(ntotset) -- minimum and maximum j bounds of each subset
!        mland -- set number of land set
!        nbdy(nx+1,ny+1) -- array of flags for identifying nodes adjacent to 
!           DEM boundary cells
!        nodeland -- indicates whether column node belongs to land
!        nset -- number of subsets found
!        set(ntotset) -- number of columns in a contiguous set
!        setflag(ntotset) -- indicates valid subset (0), or invalid by containing
!           truncated column (1), or adjacent to DEM boundary (2), or empty (-1)
!        subset(nx,ny) -- indicates set membership of DEM columns
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE GridData, ONLY: nbdy
        USE SetData, ONLY: mini,minj,maxi,maxj,set,setflag,nset
        USE ColumnData, ONLY: inscoop,subset,inn

        IMPLICIT NONE

        INTEGER, INTENT(in) :: i,j
        INTEGER :: jj,m,inscpp(4),edge(4),inscp,ibdy
        INTEGER :: nodeland,mland,ncorner,nodesum
        
        inscpp(2) = inscoop(i,j)
        inscpp(1) = inscoop(i+1,j)
        inscpp(4) = inscoop(i,j+1)
        inscpp(3) = inscoop(i+1,j+1)
        IF (ANY(inscpp.ne.0)) THEN        
          edge(2) = nbdy(i,j)
          edge(1) = nbdy(i+1,j)
          edge(4) = nbdy(i,j+1)
          edge(3) = nbdy(i+1,j+1)
          nodeland = 0       
          mland = 0
          ncorner = 0
          nodesum = 0
          ibdy = 0
          
          DO jj = 1,4
            ncorner = ncorner + 1
            inscp = inscpp(jj)
            IF (inscp.ne.0) THEN  ! corner is included in land or water set
              IF (inscp.le.nset) THEN  ! land set
                IF (mland.eq.0) mland = inscp
                IF (inscp.eq.mland) THEN  ! if node is in same subset as first column node checked
                  nodeland = nodeland + 1
                  nodesum = nodesum + ncorner
                  IF (setflag(mland).eq.0) THEN
                    IF (edge(jj).ne.0) ibdy = 1  !  node on boundary
                  END IF
                END IF             
              END IF
            END IF
          END DO

  !     If two opposite land nodes are included in 2-node column don't include as land column.  
  !     If water nodes also included mark as water column but leave node as land node for
  !     proper calculations of surrounding columns.
          IF (nodeland.eq.2.and.nodesum.eq.5) nodeland = 0
   
  !     If any land nodes assign column as a land column.
          IF (nodeland.gt.0) THEN
            m = mland
            IF (ibdy.ne.0) setflag(m) = 2
            inn(i,j) = nodeland  
            subset(i,j) = m
            set(m) = set(m) + 1
            IF (set(m).eq.1) THEN
              mini(m) = i
              maxi(m) = i
              minj(m) = j
              maxj(m) = j
            ELSE
              mini(m) = MIN(mini(m),i)
              maxi(m) = MAX(maxi(m),i)
              minj(m) = MIN(minj(m),j)
              maxj(m) = MAX(maxj(m),j) 
            END IF
          ELSE
            subset(i,j) = 0
            ! inn(i,j) = 0
          END IF
        ELSE
          subset(i,j) = 0
          ! inn(i,j) = 0
        END IF
         
        END SUBROUTINE checksets
