        SUBROUTINE Fos
    
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine calls subroutine Ordinary or Bishop, depending on
!       solution method chosen by user, to calculate the factor of safety
!       for each scoop and tracks various minimum factors of safety depending
!       on options chosen.  
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!       VARIABLES
!
!        arclength -- arc length of current 2d slice
!        area(nset) -- surface area of slide
!        area2d -- cross-sectional area of current 2d slice
!        angle -- assumed angle of slip (azimuth)
!        colfile -- flag for opening error file reporting cols < limcol
!        delxy -- DEM grid resolution (delta x, delta y)
!        delz -- z resolution of search grid
!        depthmax(nnset) -- maximum column depth in each subset
!        dtol -- tolerance on depth criterion for calculating initial radius to meet
!           minimum depth or for meeting maximum depth if required
!        fangle -- angle of minimum factor of safety for a subset
!        fangle2d -- angle of minimum FOS 2d slice for a subset
!        farclength -- arc length of minimum FOS 2d slice for a subset
!        farea2d -- area of minimum FOS 2d slice for a subset
!        farea3d2d -- area of 2d slice associated with minimum FOS 3-d scoop
!        fdepth2d -- maximum depth of minimum FOS 2d slice for a subset
!        fdepth3d2d -- maximum depth of 2d slice associated with minimum FOS 3-d scoop
!        fgrid -- minimum factor of safety at search grid point
!        filtcount(nx,ny) -- counter of number of times each column was filtered by
!           minma
!        fminma -- min m-alpha at current minimum fos slip angle
!        foscut -- factor of safety cutoff for removing slip surface from DEM
!        fosmin -- minimum factor of safety for a subset at DEM column
!        fosmin2d -- 2-d minimum factor of safety for a subset at DEM column
!        fosmin2d3d -- 3-d minimum factor of safety associated with min FOS 2-d slice
!           for a subset at DEM column
!        fosmin3d2d -- 2-d minimum factor of safety associated with min FOS 3-d scoop
!           for a subset at DEM column
!        foso2d -- 2-d factor of safety at trial angle
!        fsangle(i,j) -- slip angle of surface associated with min FOS
!        fsangle2d(nx,ny) -- array of fall angles of minimum 2d slice at each DEM column
!        fsarclength(nx,ny) -- array of arclengths of minimum 2d slice at each DEM column
!        fsarea2d(nx,ny) -- array of areas of minimum 2d slice at each DEM column
!        fsdepth(nx,ny) -- array of maximum depth of minimum scoop at each DEM cell
!        fsdepth2d(nx,ny) -- array of maximum depth of minimum 2d slice at each DEM cell
!        fsmin(i,j) -- minimum factor of safety at each DEM column
!        fsmin2d(i,j) -- 2-d minimum factor of safety at each DEM column
!        fsmin2d3d(nx,ny) -- 3-d minimum factor of safety associated with min 2d slice
!           at each DEM column
!        fsmin3d2d(nx,ny) -- 2-d minimum factor of safety associated with min 3d scoop
!           at each DEM column
!        fsrad(i,j) -- radius of sphere associated with min FOS at each DEM column
!        fsrad2d(nx,ny) -- array of radii of minimum 2d FOS slice at each DEM column
!        fsvol(i,j) -- volume of minimum fos scoop for DEM column i,j
!        fswidth2d(nx,ny) -- array of widths of minimum 2d slice at each DEM column
!        fsx(nx,ny),fsy(nx,ny),fsz(nx,ny) -- search spheroid center of minimum FOS 
!           scoop at each DEM column
!        fsx2d(nx,ny),fsy2d(nx,ny),fsz2d(nx,ny) -- search spheroid center of minimum FOS 
!           2d slice at each DEM column
!        fwidth2d -- width of minimum FOS 2d slice for a subset
!        i1,i2,j1,j1 -- array bounds for current land set
!        idepthflag -- flag indicating whether depth criteria entered as depth (1) 
!           or elevation using stratigraphic layer (2)
!        ifos2d -- flag for calculating 2-d factors of safety
!        ii1,ii2,jj1,jj2 -- array bounds for combined contiguous land and water sets
!        i2dcen,j2dcen -- i,j location of center of search spheroid intersection 
!           with DEM surface
!        isubsurf -- flag for creating 3-d file of FOS below DEM surface&
!          1=create file in ijk format, 2=create file in xyz format
!        inn(nx,ny) -- number of nodes of each column bounded by spheroid or
!          equals -1 if truncated surface
!        inscoopsave(nx+1,ny+1) -- temporary array to indicate DEM column nodes 
!           bounded by spheroid
!        iretro -- counter for number of retrogressions
!        isizefiles -- flag for writing volume, area, and depth files for min FOS at each 
!           DEM column
!        islipdirflag -- flag for automated search for minimum FOS slide direction
!        limcol -- optimal number of columns per slip surface.  Fewer columns
!           will generate error in problems.out file
!        linein(nx,ny) -- array of flags for whether 2-D slide directon line intersects
!           column
!        maxddiff(nnset) -- maximum difference between subset column bottoms and maximum depth 
!           criterion (negative if subset column bottom falls above maximum depth)
!        mset -- current land subset number
!        mcol(nx,ny) -- number of columns in minimum FOS scoop at each DEM column
!        mcol2d(nx,ny) -- number of columns in minimum 2d FOS slice at each DEM column
!        method -- B for Bishops Simplified, O for Ordinary (Fellenius) method for
!           calculation of factor of safety.
!        mflag -- flag indicating whether filter bounds were exceeded
!        minddiff(nnset) -- maximum difference between subset column bottoms and minimum depth 
!           criterion(negative if subset column bottom falls below minimum depth)
!        minma -- min absolute value of m-alpha of all iterations for a surface
!        mini(ntotset),maxi(ntotset) -- minimum and maximum i bounds of each subset
!        minj(ntotset),maxj(ntotset) -- minimum and maximum j bounds of each subset
!        mland -- set number of land set
!        ncol(nnset) -- number of columns in current scoop
!        ncol2d -- number of columns in current 2-D slice
!        nrange(nnset) -- array of flags for whether set fits volume or area criteria 
!           range
!        nretro -- number of failure retrogressions to compute
!        numdir -- number of search slip directions for each failure
!        nset -- number of land subsets found
!        ntry -- total number of slip surfaces for which FOS is calculated
!        nx -- number of DEM cells in x direction
!        ny -- number of DEM cells in y direction
!        nz -- number of nodes used for 3-D FOS values when isubsurf=1or2
!        oangle -- slip angle associated with overall minimum factor of safety
!        oangle2d -- slip angle associated with overall minimum 2d factor of safety
!        oarclength -- arc length of overall minimum 2d factor of safety
!        oarclength3d2d -- arc length of 2d surface associated with overall min 3d scoop
!        oarea -- area of overall minimum scoop
!        oarea2d -- area of overall minimum 2d slice
!        oarea2d3d -- area of 3d surface associated with overall min 2d slice
!        oarea3d2d -- area of 2d surface associated with overall min 3d scoop
!        ocol -- number of columns in minimum scoop
!        ocol2d -- number of columns in overall minimum 2-D scoop
!        ocol2d3d -- number of columns in 3d surface associated with min 2d slice
!        ocol3d2d -- number of columns in 2d surface associated with min 3d slice
!        odepth -- maximum depth of overall minimum scoop
!        odepth2d -- maximum depth of overall minimum 2-D scoop
!        odepth2d3d -- maximum depth of 3d surface associated with min 2d slice
!        odepth3d2d -- maximum depth of 2d surface associated with min 3d scoop
!        ofos -- overall minimum factor of safety
!        ofos2d -- 2-d FOS associated with overall minimum 3-d factor of safety
!        ofos2d3d -- FOS of 3d slice associated with overall minimum 2d FOS
!        ofos3d2d -- FOS of 2d slice associated with overall minimum 3d FOS
!        ofsx,ofsy,ofsz -- search spheroid center of minimum FOS scoop
!        ofsx2d,ofsy2d,ofsz2d -- search spheroid center of minimum 2d FOS scoop
!        omset -- set number of overall minimum scoop
!        oomini,oomaxi,oominj,oomaxj -- minimum and maximum bounds of all DEM cells 
!           included in any spheroid with a calculated FOS value
!        orad -- radius of spheroid associated with overall minimum factor of safety
!        orad2d -- radius of arc associated with overall minimum 2d factor of safety
!        osliparea -- slip surface area of overall min 3d scoop
!        osliparea2d3d -- slip surface area of 3d slice associated with overall minimum 2d FOS
!        ovangle -- slip angle of largest volume scoop with FOS < cutoff
!        ovarea -- area of largest volume scoop with FOS < cutoff
!        ovfos -- factor of safety of largest volume scoop with FOS < cutoff
!        ovmset -- set number of largest volume scoop with FOS < cutoff
!        ovol, oarea - volume and area of minimum FOS scoop
!        ovol2d3d -- volume of 3d surface associated with overall min 2d slice
!        ovrad -- radius of largest volume scoop with FOS < cutoff
!        ovfsx,ovfsy,ovfsz -- search spheroid center of largest volume scoop with 
!           FOS < cutoff
!        ovsliparea -- slip surface area of largest volume scoop with 
!           FOS < cutoff
!        ovvol,ovarea -- volume and area of largest volume scoop with FOS < cutoff
!        ovwt -- weight of largest volume scoop with FOS < cutoff
!        owidth2d -- width of overall min 2d FOS
!        owidth3d2d -- width of 2d surface associated with overall min 3d scoop
!        owt -- weight of overall min 3d scoop
!        owt2d3d -- weight of 3d slice associated with overall minimum 2d FOS
!        ozb(nx,ny) -- array of DEM after scoops < FOS cut off removed
!        remove -- A= all scoops < FOS cutoff will be removed from new
!           DEM.  L= only largest volume scoop < FOS cutoff will be 
!           removed. Other= the scoop with the minimum FOS will be removed.
!        rad -- search spheroid axes perpendicular to strike of slope
!        rnull -- real null value used throughout program 
!        setflag(ntotset) -- indicates valid subset (0), or invalid by containing
!           truncated node (1), or adjacent to DEM boundary (2), or empty (-1)
!        single -- flag for calculating single slip surface
!        sliparea -- area of slip surface
!        subset(nx,ny) -- indicates set membership of DEM columns
!        tol -- tolerance on primary control criterion for calculating initial 
!           radius.  (i.e. initial volume must fall between volume and volume+tol)
!        vadcrit -- indicates primary control for initial radius (1 for volume,
!           2 for area)
!        vaflag -- flag=1 if exceed maximum volume/area allowed,-1 if under volume/area
!        vmin,vmax -- minimum and maximum volume bounds for valid slip surface
!        vol(nx,ny) -- volume of column in subset
!        volume(nset) -- volume of scoop
!        weight -- weight of slide
!        width2d -- width of current 2d slice
!        xcolcount -- number of critical surfaces with total columns < limcol
!        xcount --  number of surfaces eliminated due to m-alpha limits or nonconvergence
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xdiff,ydiff -- distance from column to spheroid center
!        x2dcen,y2dcen -- perpendicular projection of x,y location of center of search spheroid  
!           intersection with DEM surface onto axis of rotation
!        x2dcolcen,y2dcolcen -- x,y location of center of search spheroid intersection 
!           with DEM surface
!        xslope(nx,ny),yslope(nx,ny) -- x and y slope vectors of DEM surface
!        xsumvol,ysumvol -- sum of DEM slope vectors times volume of columns included in slip surface
!        zmid(i,j) -- elevation of slip surface at center of intersected column
!        zbot -- elevation of bottom of 3-d fos grid array
!        zdem(i,j) -- DEM elevations
!        zfos(nx,ny,nz) -- 3-d array of minimum factor of safety below DEM surface
!        ztop -- z elev 1 node above DEM surface
!
!        OUTPUT FILES USED
!        unit filename
!          32 'fosretro_out.txt' -- file of retrogression data. Opened in Readin
!               and written in Scoops, Readin and Fos if Remove=A and nretro>0.
!          33 'filter_out.txt' -- file of data describing surfaces which fell outside
!               of user-defined m-alpha and nonconverging surfaces.  
!               Opened and written in FOS if filter = 1.
!          34 'ncolerr_out.txt' -- file of information about critical surfaces with
!               fewer columns than user-defined limit, limcol. Opened and
!               written in Fos if colfile = 1.
!          40 'spheresltcut_out.txt' --  file of sphere center coordinates, radius,
!               fall angle, # of columns in scoop, vol, and factor of safety of all
!               surfaces with F<foscut. Opened in readin and written in fos if remove='A'.
!
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,nullhi,rnull,outputdir,filin,bfile,nfile
        USE GridData, ONLY: xcen,ycen,zcen,zdem,angle,delz,rad,nz,delxy,pi,twoPi,iretro,&
                           lengthunits,xll,yll,Piover180,invPiover180,angslope,cellarea
        USE SetData, ONLY: area,ncol,nrange,mini,minj,maxi,maxj,setflag,colfile,&
                           sliparea,weight,volume,nset,mset,xcolcount,i1,i2,j1,j2,&
                           cosang,sinang
        USE SearchData, ONLY : linenum,nrec,islipdirflag
        USE FOSData
        USE Columndata, ONLY: i2dcen,j2dcen,x2dcen,y2dcen,subset,zmid,xslope,yslope,vol,inn
        USE DepthData, ONLY: depthmax,idepthflag
      
        IMPLICIT NONE
        
        INTERFACE
          SUBROUTINE SubsetError(ierr)
            INTEGER, INTENT(in) :: ierr 
          END SUBROUTINE SubsetError
          
          SUBROUTINE Ordinary
          END SUBROUTINE Ordinary

          SUBROUTINE Bishop(mflag)
            INTEGER, INTENT(out) :: mflag
          END SUBROUTINE Bishop

          SUBROUTINE MinFOSangle
          END SUBROUTINE MinFOSangle
        END INTERFACE
        
        INTEGER :: mflag,i,j,k,calcangle,ierr

        REAL(dp) :: fosmin,ztop,z,fminma,xdiff,ydiff,fdepth3d2d
        REAL(dp) :: fangle,farea2d,farclength2d,fncol2d,fdepth2d
        REAL(dp) :: fosmin2d,fosmin2d3d,fosmin3d2d,fangle2d,fwidth2d
        REAL(dp) :: fwidth3d2d,farea3d2d,farclength3d2d,fncol3d2d
        REAL(dp) :: x2dcolcen,y2dcolcen,yprime,midbase
        REAL(dp) :: felfosmin,xsumvol,ysumvol,vv,xslp,yslp
       
        CHARACTER(10) :: cnum
        CHARACTER(4)  :: fosmeth
        CHARACTER(1) :: charetro
        
!     Initialize variables
        cnum = '0123456789'
        charetro = cnum(iretro+1:iretro+1)
        ierr = 0
        mflag = 0
        felfoso = 0.0_dp
        nangle = 0
        newangle = 0
        IF (single.eq.0.or.angle.gt.360._dp) THEN
          calcangle = 1
        ELSE
          calcangle = 0
        END IF
      
!     Loop through all sets in current slip surface
        DO mset = 1,nset
!     If not valid subset, go on to next one.
          IF ((nrange(mset).eq.0.and.single.eq.0).or.setflag(mset).ne.0) CYCLE 
          fosmin = nullhi
          foso = nullhi                    
          
!    Set loop range as minimum and maximum locations of columns
!    in set mset.              
          j1 = minj(mset)
          j2 = maxj(mset)
          i1 = mini(mset)
          i2 = maxi(mset)        

          IF (calcangle.ne.0) THEN
!      Calculate fall line if not given as part of single search or
!      if comparison fall line needed for ellipsoids.  If fall line
!      is given for single search, check whether surface is flat.
            xsumvol = 0.0_dp
            ysumvol = 0.0_dp       
            DO j = j1,j2
              DO i = i1,i2           
!     Sum DEM surface slope vectors to find resultant slope 
!     vectors using volume weighted slopes.
                IF (subset(i,j).eq.mset) THEN
                  xslp = xslope(i,j)
                  yslp = yslope(i,j)
                  vv = vol(i,j)
                  IF (xslp.ne.0.0_dp.or.yslp.ne.0.0_dp) THEN
                    xsumvol = xsumvol + xslp*vv
                    ysumvol = ysumvol + yslp*vv
                  END IF
                END IF  
              END DO
            END DO
!     If surface is flat, move to next set.
            IF (ysumvol.eq.0.0_dp.and.xsumvol.eq.0.0_dp) CYCLE
!     Estimate fall line (convert to degrees)for spheres or comparison fall line for ellipsoids.
            ! angslope = FastAtan2(ysumvol,xsumvol)
            angslope = ATAN2(ysumvol,xsumvol)
            IF (angslope.lt.0.0_dp) angslope = angslope + twopi
          ELSE  !  slip angle is given for single surface, convert to radians
            angslope = angle * Piover180
          END IF

!     To calculate 2D center of rotation find approximate center of all intersected columns using
!     max and min column locations. Add 1 to find center of min and max nodes.	        
          IF (ifos2d.ne.0) THEN 
            fosmin2d = nullhi 
            i2dcen = INT(REAL(i1+i2+1)*0.5_dp+0.001_dp*delxy)
            j2dcen = INT(REAL(j1+j2+1)*0.5_dp+0.001_dp*delxy) 
            x2dcolcen = (REAL(i1+i2-1)*0.5_dp) * delxy 
            y2dcolcen = (REAL(j1+j2-1)*0.5_dp) * delxy 
            xdiff = x2dcolcen - xcen
            ydiff = y2dcolcen - ycen
          END IF
          
!     Find minimum factors
          cosang = COS(angslope)
          sinang = SIN(angslope)
          IF (calcangle.eq.1) THEN
            angle = angslope * invPiover180
            IF (angle.lt.0.0_dp) angle = angle + 360.0_dp
          END IF
                
          DO !  Loop on FOS calculation if iterating to find minimum FOS slide angle 
            IF (ifos2d.ne.0) THEN 
!     Project center of intersected columns onto axis of rotation
!     to estimate 2D center of rotation. This center will be used
!     for 2-D FOS solutions such that all 2-D fall lines will
!     pass through this point.
              yprime = xdiff*(-sinang)+ydiff*cosang
              x2dcen = -yprime*sinang + xcen
              y2dcen = yprime*cosang + ycen
            END IF
            
            SELECT CASE (method)
              CASE ('o')
                CALL Ordinary                          

              CASE ('b') 
                CALL Bishop(mflag)
                                            
            END SELECT

!     Count how many FOS trials.            
            ntry = ntry + 1
          
!     If surface fell outside of m-alpha or FOS limits or did not converge 
!     in Bishop write scoop parameters to filter.out file.
            IF (mflag.ne.0) THEN
!     If filter file not yet opened, open it and write header lines. Initialize 
!     count of filtered search nodes.
              IF (mfile.eq.0) THEN
                IF (iretro.gt.0) THEN 
                  IF (outputdir.ne.'') THEN
                    OPEN (33,STATUS='replace',file = outputdir(1:LEN_TRIM(outputdir))//&                 
                       filin(bfile:nfile)//'_filter'//charetro//'_out.txt')
                   ELSE
                     OPEN (33,STATUS='replace',file = &                 
                       filin(bfile:nfile)//'_filter'//charetro//'_out.txt')
                   END IF
                ELSE
                  IF (outputdir.ne.'') THEN
                    OPEN (33,STATUS='replace',file = outputdir(1:LEN_TRIM(outputdir))//&
                       filin(bfile:nfile)//'_filter_out.txt') 
                  ELSE
                    OPEN (33,STATUS='replace',file = &
                       filin(bfile:nfile)//'_filter_out.txt')
                  END IF
                END IF
                WRITE (33,1500)
                WRITE (33,1510)
                IF (absminma.ne.0) WRITE (33,1520) absminma 
                WRITE (33,1540)
                IF (LEN_TRIM(lengthunits).eq.1) THEN
                  WRITE (33,1551) lengthunits,lengthunits,lengthunits,lengthunits
                ELSE
                  IF (LEN_TRIM(lengthunits).eq.2) THEN
                    WRITE (33,1552) lengthunits,lengthunits,lengthunits,lengthunits
                  ELSE 
                    WRITE (33,1550)
                  END IF  
                END IF                   
                mfile = 1
                xcount = 0
              END IF
!    Write filtered search center data and calculated FOS to filter file.
              IF (method.eq.'b') THEN
                WRITE (33,1560) xcen+xll,ycen+yll,zcen,rad,angle,minma,foso,felfoso
              ELSE
                WRITE (33,1620) xcen+xll,ycen+yll,zcen,rad,angle,foso
              END IF
!    Set calculated FOS (flagged for m-apha) to null value so they will not be compared to
!    other factors of safety.
              IF (foso.ne.100.0_dp) foso = 111.0_dp
              IF (ifos2d.ne.0) THEN
                IF (foso2d.ne.100.0_dp) foso2d = 111.0_dp
              END IF
              xcount = xcount + 1
            END IF  !  IF (mflag.ne.0)
          
!     If calculating 2-D factors of safety, save minimum 2-D factor of safety
!     and associated 3-D factor of safety of all angles.  Angle for minimum 2-D
!     may differ from minimum 3-D so keep in angle iteration loop.
            IF (ifos2d.ne.0) THEN
              IF (foso2d.lt.fosmin2d) THEN
                fosmin2d = foso2d
                fosmin2d3d = foso
                fangle2d = angle 
                fwidth2d = width2d
                farea2d = area2d
                farclength2d = arclength
                fncol2d = ncol2d
                IF (idepthflag.ne.0.or.isizefiles.ne.0) fdepth2d = depth2d
              END IF
            END IF   ! IF (ifos2d.eq.1)

            newangle = 0 
!     If iterating to find minimum FOS angle calculate next angle to try
            IF (islipdirflag.ne.0) THEN
              IF (mflag.eq.0.and.foso.lt.100._dp) THEN
                nangle = nangle + 1
                CALL MinFOSangle
              END IF
            END IF
     
            IF (newangle.eq.0) EXIT
            
          END DO  !  loop on minimum FOS slide angle search            

!     If removing all surfaces with FOS<foscut, save bottom elevation to ozb.
          IF (remove.eq.'A') THEN
            IF (foso.lt.foscut) THEN
              IF (idepthflag.eq.0.or.isizefiles.ne.0) THEN
                WRITE (40,1400) xcen+xll, ycen+yll,zcen,rad,angle,volume(mset),area(mset),foso
                IF (nretro.gt.0) WRITE (32,1300) foso,xcen+xll,&
                     ycen+yll,zcen,rad,angle,volume(mset),area(mset)
              ELSE
                WRITE (40,1401) xcen+xll, ycen+yll,zcen,rad,angle,volume(mset),area(mset),depthmax(mset),foso
                IF (nretro.gt.0) WRITE (32,1301) foso,xcen+xll,&
                     ycen+yll,zcen,rad,angle,volume(mset),area(mset),depthmax(mset)
              END IF
              DO i=i1,i2
                DO j=j1,j2
                  IF (subset(i,j).eq.mset) THEN
                    IF (inn(i,j).eq.4) THEN
                      ozb(i,j) = zmid(i,j)
                    ELSE
!     For partial columns find new elevation of full cell to match volume removed
                      ozb(i,j) = zdem(i,j) - vol(i,j)/cellarea
                    END IF
                  END IF
                END DO
              END DO
            END IF
          END IF  !  IF (remove.eq.'A')

!     Save minimum 3-D factors of safety and associated 2-D values if necessary
!     for all angles tried at this search spheroid.
          IF (foso.lt.fosmin) THEN
            fosmin = foso
            fangle = angle
            IF (method.eq.'b') THEN
              felfosmin = felfoso
              fminma = minma
            END IF                
            IF (ifos2d.ne.0) THEN
              fosmin3d2d = foso2d  
              farea3d2d = area2d
              farclength3d2d = arclength
              fwidth3d2d = width2d
              fncol3d2d = ncol2d
              IF (idepthflag.ne.0.or.isizefiles.ne.0) fdepth3d2d = depth2d
            END IF  
          END IF  ! IF (foso.lt.fosmin)   
            
          IF (single.eq.0) THEN 
!     If # columns less than limcol, write search location to ncolerr.out.            
            IF (ncol(mset).lt.limcol) THEN
              IF (colfile.eq.0) THEN  
                IF (iretro.gt.0) THEN
                  IF (outputdir.ne.'') THEN                
                    OPEN (34,STATUS='replace',file = outputdir(1:LEN_TRIM(outputdir))//&
                     filin(bfile:nfile)//'_ncolerr'//charetro//'_out.txt')
                  ELSE
                    OPEN (34,STATUS='replace',file = &
                     filin(bfile:nfile)//'_ncolerr'//charetro//'_out.txt')
                  END IF
                ELSE
                  IF (outputdir.ne.'') THEN
                    OPEN (34,STATUS='replace',file = outputdir(1:LEN_TRIM(outputdir))//&
                        filin(bfile:nfile)//'_ncolerr_out.txt')
                  ELSE
                    OPEN (34,STATUS='replace',file = &
                        filin(bfile:nfile)//'_ncolerr_out.txt')
                  END IF
                END IF
                WRITE (34,1000)
                IF (method.eq.'b') THEN
                  fosmeth = 'Bish'
                ELSE
                  fosmeth = 'Ord'
                END IF
                IF (idepthflag.eq.0.or.isizefiles.ne.0) THEN
                  IF (LEN_TRIM(lengthunits).eq.1) THEN
                      WRITE (34,1110) lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,fosmeth
                  ELSE
                     IF (LEN_TRIM(lengthunits).eq.2) THEN
                        WRITE (34,1120) lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,fosmeth
                     ELSE
                        WRITE (34,1100) fosmeth
                    END IF  
                  END IF
                ELSE
                  IF (LEN_TRIM(lengthunits).eq.1) THEN
                      WRITE (34,1111) lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,fosmeth
                  ELSE
                     IF (LEN_TRIM(lengthunits).eq.2) THEN
                        WRITE (34,1121) lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,lengthunits,fosmeth
                     ELSE
                        WRITE (34,1101) fosmeth
                    END IF  
                  END IF
                END IF
                colfile = 1
                xcolcount = 0
              END IF
              xcolcount = xcolcount + 1
              IF (idepthflag.eq.0.or.isizefiles.ne.0) THEN
                WRITE (34,1200) ncol(mset),xcen+xll,ycen+yll,zcen,rad,volume(mset),area(mset),foso
              ELSE
                WRITE (34,1201) ncol(mset),xcen+xll,ycen+yll,zcen,rad,volume(mset),area(mset),depthmax(mset),foso
              END IF
            END IF  ! IF (ncol(mset).lt.limcol)
          END IF  ! IF (single.eq.0)

!     Keep track of minimum factor of safety at each search center        
          IF (fosmin.le.fgrid) fgrid = fosmin       

          IF (fosmin.eq.nullhi) CYCLE

!     Check if 3-D factor of safety is less than previous minimum 
!     for each DEM column.
          DO j= j1,j2
            DO i = i1,i2
              IF (subset(i,j).eq.mset) THEN
                IF (fosmin.lt.fsmin(i,j)) THEN
                  IF (icritlattice.ne.0) linenum(i,j) = nrec
                  fsmin(i,j) = fosmin
                  IF (method.eq.'b') felfsmin(i,j) = felfosmin
                  IF (ifos2d.ne.0) fsmin3d2d(i,j) = fosmin3d2d
                  fsangle(i,j) = fangle
                  fsrad(i,j) = rad
                  fsx(i,j) = xcen
                  fsy(i,j) = ycen
                  fsz(i,j) = zcen
                  mcol(i,j) = ncol(mset)
                  fsvol(i,j) = volume(mset)
                  fsarea(i,j) = area(mset)
                  IF (idepthflag.ne.0.or.isizefiles.ne.0) fsdepth(i,j) = depthmax(mset)
                END IF  ! IF (fosmin.lt.fsmin(i,j))
                
!     Track minimum 2-D FOS as well.
                IF (ifos2d.ne.0) THEN
                  IF ((linein(i,j).ne.0).and.(fosmin2d.lt.fsmin2d(i,j))) THEN
                    fsmin2d(i,j) = fosmin2d
                    fsmin2d3d(i,j) = fosmin2d3d
                    fsrad2d(i,j) = rad
                    fsangle2d(i,j) = fangle2d
                    fsx2d(i,j) = xcen
                    fsy2d(i,j) = ycen
                    fsz2d(i,j) = zcen
                    mcol2d(i,j) = fncol2d
                    fsarea2d(i,j) = farea2d
                    fswidth2d(i,j) = fwidth2d
                    fsarclength(i,j) = farclength2d
                    IF (idepthflag.ne.0.or.isizefiles.ne.0) fsdepth2d(i,j) = fdepth2d
                  END IF
                END IF  ! IF (ifos2d.eq.1)
!     Keep running sum of number of times each column is filtered.               
                IF (mflag.ne.0) filtcount(i,j) = filtcount(i,j) + 1                 

                IF (isubsurf.ne.0) THEN    
!     Find z stopping point at 5 nodes above actual DEM.
                  ztop = zdem(i,j) + 5.0_dp*delz
                  IF (inn(i,j).eq.4) THEN
                    midbase = zmid(i,j)
                  ELSE
!     For partial columns find new elevation of full cell to match volume removed
                    midbase = zdem(i,j) - vol(i,j)/cellarea
                    zmid(i,j) = midbase
                  END IF
!     Now check z values above base of trial surface.
                  DO k = 1,nz
                    z = zbot + REAL(k-1,dp)*delz
!     Compare FOS, replace if z >= zmid and  z <= stop elev
!     and FOS less. 
                    IF (z.ge.midbase.and.z.le.ztop) THEN
                      IF (fosmin.lt.zfos(i,j,k)) zfos(i,j,k)=fosmin
                    END IF
                  END DO 
                END IF  ! IF (isubsurf.ne.0)
              END IF   ! IF (subset(i,j).eq.mset) 
            END DO  ! loop on i
          END DO  !  loop on j

          IF (isubsurf.ne.0) THEN
            DO j= j1,j2
              DO i = i1,i2
                IF (subset(i,j).eq.mset) THEN
!     Find z stopping point at 5 nodes above actual DEM.
                  ztop = zdem(i,j) + 5.0_dp*delz
                  midbase = zmid(i,j)
!     Now check z values above base of trial surface.
                  DO k = 1,nz
                    z = zbot + REAL(k-1,dp)*delz
!     Compare FOS, replace if z >= zmid and  z <= stop elev
!     and FOS less.
                    IF (z.ge.midbase.and.z.le.ztop) THEN
                      IF (fosmin.lt.zfos(i,j,k)) zfos(i,j,k)=fosmin
                    END IF
                  END DO 
                END IF   ! IF (subset(i,j).eq.mset) 
              END DO  ! loop on i
            END DO  !  loop on j
          END IF  ! IF (isubsurf.ne.0)

          IF (fosmin.lt.0.0) THEN
            PRINT *,'WARNING - fosmin<0!!',fosmin,xcen+xll,ycen+xll,zcen,rad
            WRITE (39,*) 'WARNING - fosmin<0!!',fosmin,xcen+xll,ycen+xll,zcen,rad
          END IF

!     Check if overall factor of safety is minimum.
          IF (fosmin.lt.ofos) THEN
            ofos = fosmin
            IF (method.eq.'b') felminfos = felfosmin
            orad = rad
            oangle = fangle
            ofsx = xcen
            ofsy = ycen
            ofsz = zcen
            ovol = volume(mset)
            oarea = area(mset)
            ocol = ncol(mset)
            omset = mset
            IF (idepthflag.ne.0.or.isizefiles.ne.0) odepth = depthmax(mset)
            IF (method.eq.'b') ominma = fminma
            owt = weight
            osliparea = sliparea
            IF (ifos2d.ne.0) THEN
              ofos3d2d = fosmin3d2d
              oarea3d2d = farea3d2d
              oarclength3d2d = farclength3d2d
              owidth3d2d = fwidth3d2d
              ocol3d2d = fncol3d2d
              IF (idepthflag.ne.0.or.isizefiles.eq.1) odepth3d2d = fdepth3d2d
            END IF         
          END IF
          IF (ifos2d.ne.0) THEN
            IF (fosmin2d.lt.ofos2d) THEN
              ofos2d = fosmin2d
              orad2d = rad
              oangle2d = fangle2d
              ofsx2d = xcen
              ofsy2d = ycen
              ofsz2d = zcen
              oarea2d = farea2d
              oarclength = farclength2d
              owidth2d = fwidth2d
              ocol2d = fncol2d
              ofos2d3d = fosmin2d3d
              oarea2d3d = area(mset)
              ovol2d3d = volume(mset)
              ocol2d3d = ncol(mset)
              owt2d3d = weight
              osliparea2d3d = sliparea
              IF (idepthflag.ne.0.or.isizefiles.ne.0) THEN
                odepth2d = fdepth2d
                odepth2d3d = depthmax(mset)
              END IF
            END IF
          END IF

!     Keep track of largest vol scoop with FOS < cut off (option remove = L).
          IF (remove.eq.'L') THEN
            IF (fosmin.lt.foscut.and.volume(mset).gt.ovvol) THEN
              ovfos = fosmin
              ovrad = rad
              ovangle = fangle
              ovfsx = xcen
              ovfsy = ycen
              ovfsz = zcen
              ovvol = volume(mset)
              ovarea = area(mset)
              ovmset = mset
              ovwt = weight
              ovsliparea = sliparea
              IF (idepthflag.ne.0.or.isizefiles.ne.0) ovdepth = depthmax(mset)
            END IF
          END IF      
        END DO  ! loop on subsets mset
  
1000   FORMAT ('Trial surfaces with columns < limcol')
1100   FORMAT ('# cols       xcen          ycen         zcen      radius    volume      area       F_',A4)
1110   FORMAT ('# cols     xcen_',A1,'          ycen_',A1,'       zcen_',A1,'   radius_',A1,' volume_',A1,'^3  area_',A1,&
                            '^2  ','   F_',A4)
1120   FORMAT ('# cols     xcen_',A2,'         ycen_',A2,'      zcen_',A2,'  radius_',A2,' volume_',A2,'^3 area_',A2,&
                          '^2 ','  F_',A4)
1101   FORMAT ('# cols       xcen          ycen         zcen      radius    volume      area       depth      F_',A4)
1111   FORMAT ('# cols     xcen_',A1,'          ycen_',A1,'       zcen_',A1,'   radius_',A1,' volume_',A1,'^3  area_',A1,&
                            '^2    depth_',A1,'     F_',A4)
1121   FORMAT ('# cols     xcen_',A2,'         ycen_',A2,'      zcen_',A2,'  radius_',A2,' volume_',A2,'^3 area_',A2,&
                          '^2   depth_',A2,'    F_',A4)
1200   FORMAT (i5,2f16.4,2f10.3,2es11.3,2x,f8.4)
1201   FORMAT (i5,2f16.4,2f10.3,3es11.3,2x,f8.4)     
1290   FORMAT (3(' ',f10.2),' ',f11.4,' ',f8.4,' ',2es11.3,f7.4)
1300   FORMAT (f7.4,2f16.4,' ',f10.2,' ',f11.4,' ',f8.4,' ',2es11.3)
1301   FORMAT (f7.4,2f16.4,' ',f10.2,' ',f11.4,' ',f8.4,' ',3es11.3)
1400   FORMAT (2f16.4,' ',f10.2,' ',f11.4,' ',f8.4,' ',2es11.3,f7.4)
1401   FORMAT (2f16.4,' ',f10.2,' ',f11.4,' ',f8.4,' ',3es11.3,f7.4)
1500   FORMAT ('This file contains data for trial slip surfaces filtered by&
                & absminma')
1510   FORMAT ( ' and nonconverging surfaces.')
1520   FORMAT ('Minimum absolute value m-alpha: ',f10.5) 
1540   FORMAT ('Converging F are written here, but not used in search for minimum F.')
1550   FORMAT ('           xcen           ycen      zcen    radius     angle     minma      F_Bish    F_Ord')     
1551   FORMAT ('         xcen_',A1,'         ycen_',A1,'    zcen_',A1,'   radius_',A1,'   angle     minma     &
              &F_Bish      F_Ord')
1552   FORMAT ('        xcen_',A2,'         ycen_',A2,'   zcen_',A2,'   radius_',A2,'   angle     minma     F_Bish    F_Ord')     
1560   FORMAT (2f16.4,3f10.3,es11.3,2f10.3) 
1620   FORMAT (2f16.4,3f10.3,f10.3)
       
       END SUBROUTINE Fos
