        SUBROUTINE Nodeinout(inode,jnode) 
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine checks whether node is contained by spheroid to find
!       bounds for flood fill routine. Also checks if land node is adjacent 
!       to a water node.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!        Called by SubsetFill
!
!        VARIABLES
!
!        delxy -- DEM grid resolution (delta x, delta y)
!        icont -- flag for whether node is included in spheroid 
!        inode,jnode -- i and j of current DEM column
!        mz -- elevation difference from spheroid center to spheroid border at 
!           current i,j location
!        mz2 -- square of mz
!        radsq -- square of search spheroid axes perpendicular to strike of slope
!        x,y -- location of column in DEM coordinates
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xprime,yprime,zprime -- x,y, and z coordinates translated relative to spheroid center
!        zdemnodes(nx+1,ny+1) -- DEM node elevations averaged from surrounding cells
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,rnull2
        USE GridData, ONLY: xcen,ycen,zcen,radsq,zdemnodes,delxy
        USE SetData, ONLY: icont
        
        IMPLICIT NONE

        INTEGER, INTENT(in) :: inode,jnode
                
        REAL(dp) :: mz2,mz,x,y,spherbottom,xprime,yprime
  
        icont = 0
        x = REAL(inode-1,dp)*delxy
        y = REAL(jnode-1,dp)*delxy
        xprime = x-xcen
        yprime = y-ycen
        mz2 = radsq-(xprime*xprime+yprime*yprime)

!     Check whether DEM node is intersected by spheroid. mz2 will
!     be less than zero if x or y of node is outside spheroid.     
        IF (mz2 .gt. 0.0_dp) THEN
          mz = sqrt(mz2)
          spherbottom = zcen - mz
!     If subset seed is designated as a land node determine whether
!     DEM elevation is above bottom of spheroid at x,y location
          IF (zdemnodes(inode,jnode).ge.spherbottom) icont = 1
        END IF  !  IF (mz2 .gt. 0.0_dp)
            
        END SUBROUTINE
        