        SUBROUTINE NodeTest(inode,jnode) 
        
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!       This subroutine checks whether node is part of current subset and
!       assigns spheroid bottom elevation to zb array.
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
!
!        Called by FloodFill and EllipWaterSets
!
!        VARIABLES
!
!        delxy -- DEM grid resolution (delta x, delta y)
!        icont -- flag for whether node is included in spheroid 
!        inode,jnode -- i and j of current DEM column
!        inscoop(nx,ny) -- indicates subset membership of DEM column nodes
!        mz -- elevation difference from spheroid center to spheroid border at 
!           current i,j location
!        mz2 -- square of mz
!        nodecount -- number of land nodes contained in spheroid
!        numset -- current set number
!        omini,omaxi,ominj,omaxj -- minimum and maximum bounds of all sets included
!           in search spheroid
!        radsq -- square of radius
!        xcen,ycen,zcen -- center of search spheroid relative to DEM origin
!        xprime,yprime,zprime -- x,y, and z coordinates translated relative to spheroid center
!        zb(nx+1,ny+1) -- base of slip surface at each node (not column)
!        zdemnodes(nx+1,ny+1) -- DEM node elevations averaged from surrounding cells
!xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

        USE CommonData, ONLY: dp,rnull,rnull2
        USE GridData, ONLY: xcen,ycen,zcen,radsq,zdemnodes,delxy
        USE ColumnData, ONLY: zb,inscoop
        USE SetData, ONLY: omini,omaxi,ominj,omaxj,set,numset,nodecount,icont
        
        IMPLICIT NONE

        INTEGER, INTENT(in) :: inode,jnode
                
        REAL(dp) :: mz2,mz,zd,spherbottom
        REAL(dp) :: xprime,yprime,xppsq,yppsq
  
        icont = 0
        xprime = REAL(inode-1,dp)*delxy - xcen
        yprime = REAL(jnode-1,dp)*delxy - ycen
        xppsq = xprime * xprime
        yppsq = yprime * yprime
        mz2 = radsq - (xppsq + yppsq)

!     Check whether DEM node is intersected by spheroid. mz2 will
!     be less than zero if x or y of node is outside spheroid.     
        IF (mz2 .gt. 0.0_dp) THEN
          mz = sqrt(mz2) 
          zd = zdemnodes(inode,jnode)
          spherbottom = zcen - mz
!     If subset seed is designated as a land node determine whether
!     DEM elevation is above bottom of spheroid at x,y location.
          IF (zd.ge.spherbottom) THEN
            icont = 1
            inscoop(inode,jnode) = numset
            zb(inode,jnode) = spherbottom
!     Keep track of min and max array bounds for nodes.
            omini = MIN(omini,inode)
            omaxi = MAX(omaxi,inode)
            omaxj = MAX(omaxj,jnode)
            ominj = MIN(ominj,jnode)
            nodecount = nodecount + 1
          ELSE
            zb(inode,jnode) = rnull         
          END IF
         
!     Accumulate number of nodes in set
          IF (icont.ne.0) THEN
            set(numset) = set(numset) + 1
          END IF
        ELSE
          zb(inode,jnode) = rnull
        END IF  !  IF (mz2 .gt. 0.0_dp)
            
        END SUBROUTINE NodeTest
        